
/* Name: main.c
 * Project: Presentation remote - transmitter
 * Author: jiristepanovsky.cz
 * Creation Date: 2013-12-1
 */

// 9.6Mhz internal oscillator, devision by 8 disabled
#define F_CPU 9600000UL

#include <avr/io.h>
#include <util/delay.h>

#define DDR DDRB //PORTB
#define PORT PORTB //PORTB
#define PIN PINB //PORTB

#define TX 0b00001000 //PORTB3
#define SET_TX() { PORT |= TX; } //Set PORTB3
#define RESET_TX() { PORT &= ~TX; } //Reset PORTB3

// Buttons
#define LEFT 0b00000010 //PORTB1
#define RIGHT 0b00010000 //PORTB4
#define A 0b00000001 //PORTB0
#define B 0b00000100 //PORTB2


//############################################

unsigned char high = 0;

void transferBit(unsigned char bit){
    if (bit) {
        if(high){
            RESET_TX();
            _delay_us(300);
        } else {
            SET_TX();
            _delay_us(300);
        }
        high = (high)? 0 : 1;
    } else {
        if(high){
            RESET_TX();
            _delay_us(110);
            SET_TX();
            _delay_us(110);
        } else {
            SET_TX();
            _delay_us(110);
            RESET_TX();
            _delay_us(110);
        }
    }
}

void sendCommand(unsigned short id, unsigned char code){
    signed char i;
    while(1){
        SET_TX();
        _delay_us(500); // 550
        high = 1;
        for (i = 7; i >= 0; i--) { // transmit remoteID
            transferBit((id>>8)&(1<<i));
        }
        for (i = 7; i >= 0; i--) { // transmit remoteID
            transferBit(id&(1<<i));
        }
        for (i = 6; i >= 0; i--) { // transmit keycode
            transferBit(code&(1<<i));
        }
        RESET_TX();
        _delay_us(8);
    }
}


int main(void)
{

	// One of buttons pressed
    unsigned char buttons = 0;

    // Enable pull-up rezistors on button pins
	PORT = LEFT | RIGHT | A | B;

    //Set TX pin as output
    DDR = TX;

    // Wait a little while
    _delay_us(10);

	if(PIN & LEFT)
        buttons |= 1<<0;
    if(PIN & RIGHT)
        buttons |= 1<<1;
    if(PIN & A)
        buttons |= 1<<2;
    if(PIN & B)
        buttons |= 1<<3;
    PORT = 0;

	// ID = 6400; 19303; 10550; 8500; 7400
    // Keys = #1: 0, #2: 96, #3: 120, #4: 24, #5: 80, #6: 48, #7: 108, #8: 12, #9: 72; #10: 40, #OFF: 106
        
    if(buttons == 1){
        // Only upper left button pressed - light on/off
        sendCommand(19303, 120); // #3
    }
        
    if(buttons == 2){
        // Only upper right button pressed - led strip on/off
        sendCommand(19303, 96); // #2
    }
        
    if(buttons == 4){
        // Only bottom left button pressed - all off
        sendCommand(19303, 106); // OFF
    }
    if(buttons == 8){
        // Only bottom right pressed - something
        sendCommand(19303, 24); // #4
    }

    // Do nothing
   	while(1);
}
